//
// datePicker.js (date picker)
//
//

//
// KHP-RC, revise if we add support for hours, minutes and seconds in the calendar picker,
// repectively for date filters such as 18/Jan/2006 13:20:10, 18jan2006_132010
//

var datePicker = {
	
	dfMonths: ['jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec'],
	
	relativeDateUnits: ['year', 'quarter', 'month', 'week', 'day'], // hour, minute and second is not supported in v8
	
	dateUnitLabelsPlural: {
		year: langVar('lang_stats.date_picker.years'),
		quarter: langVar('lang_stats.date_picker.quarters'),
		month: langVar('lang_stats.date_picker.months'),
		week: langVar('lang_stats.date_picker.weeks'),
		day: langVar('lang_stats.date_picker.days')
	},
	
	panel: null,
	// tabs: null,
	// tabSpaceFixed: false,
	// isDateRange: false,
	
	activePicker: '', // dp1 | dp2 | dp3
	
	calendar: null, // calendar date picker object
	relative: null, // relative date picker object
	
	earliest: {
		year: 0,
		month: 0,
		day: 0
	},
	
	latest: {
		year: 0,
		month: 0,
		day: 0
	},
	
	dp1: { // date picker 1, start date
		picker: 'dp1',
		dayOffset: 0, // the dayOffset of the active month
		dateUnit: '', // the active dateUnit: year | quarter | month | day | ''
		year: -1, // We use -1 as default and for inactive values to check whether a year, quarter, month or day is selected
		quarter: -1,
		month: -1,
		day: -1
	},
	
	dp2: { // date picker 2, end date
		picker: 'dp2',
		dayOffset: 0,
		dateUnit: '',
		year: -1,
		quarter: -1,
		month: -1,
		day: -1
	},
	
	dp3: { // date picker 3, relative date
		// We set default values so that we don't need separate initialization for dp3
		picker: 'dp3',
		isActive: false, 	// deteremines if we have an active selection. If true
							// all items are selected in full color. If false we use the entire date range,
							// though the specified items are semi selected.
						
		type: 'recent', // recent | last
		dateUnitCount: 1, // must be always an integer >= 1
		dateUnit: 'year' // the active dateUnit: year | quarter | month | week | day | hour | minute | second
	},
	
	clearEndDateBtn: null,
		
	init: function() {
	
		var panelObj = {
			panelId: 'dp:panel',
			panelClassName: 'panel-50',
			panelHeaderLabel: langVar('lang_stats.date_picker.date_picker'),
			left: 20,
			top: 50,
			zIndex: 20,
			isCover: true,
			closeEvent: datePicker.close
		};
		
		datePicker.panel = new util.Panel3(panelObj);
		
		// Init tabs
		// datePicker.tabs = new util.Tabs2(['date_picker:start_date_tab', 'date_picker:end_date_tab'], datePicker.tabActivated);
		
		
		//
		// Get earliest and latest data info
		// 
		
		var obj = util.salangDateToSimpleDateObject(reportInfo.earliestDate);
		var earliest = datePicker.earliest;
		earliest.year = obj.year;
		earliest.month = obj.month;
		earliest.day = obj.day;
		
		obj = util.salangDateToSimpleDateObject(reportInfo.latestDate);
		var latest = datePicker.latest;
		latest.year = obj.year;
		latest.month = obj.month;
		latest.day = obj.day;
		
		// util.showObject(earliest);
		// util.showObject(latest);
		
		
		//
		// Create the date picker main form
		//
		
		datePicker.build();
		
		
		//
		// Create the calendar picker object (builds HTML too)
		//
		datePicker.calendar = new datePickerUtil.Calendar(earliest, latest, 'dp:inner_pickers_container');
		
		//
		// Create the relative picker object (builds HTML too)
		//
		datePicker.relative = new datePickerUtil.Relative('dp:inner_pickers_container', datePicker.dp3);
		
		
		//
		// Assign a single event per calendar picker and relative date picker
		//
		yEvent.addListener('dp:calendar_picker', 'click', datePicker.calendarPickerItemActivated);
		
		yEvent.addListener('dp:relative_date_display:date_unit_count', 'keyup', datePicker.relativePickerDateUnitCountInput);
		yEvent.addListener('dp:relative_picker', 'click', datePicker.relativePickerItemActivated);


		// Init buttons
		
		yEvent.addListener(['dp:dp1:btn', 'dp:dp2:btn', 'dp:dp3:btn'], 'click', datePicker.setPickerByEvent);
		yEvent.addListener('dp:clear_picker_date:btn', 'click', datePicker.clearPickerDate);
		
		yEvent.addListener('dp:apply_btn', 'click', datePicker.applyDateFilter);
		yEvent.addListener('dp:cancel_btn', 'click', datePicker.close);
	},
	
	initCalendarDpObject: function(dpName, decomposition) {
		
		// Initializes dp1 and dp2 values according the active date filter via 
		// the decomposition object.
		
		var dp = datePicker[dpName];
		
		// We reset the dp object because it could be already set
		dp.dayOffset = 0;
		dp.dateUnit = '';
		dp.year = -1;
		dp.quarter = -1;
		dp.month = -1;
		dp.day = -1;
		
		if (decomposition) {
			
			dp.year = decomposition.year;
			dp.quarter = decomposition.quarter;
			dp.month = decomposition.month;
			dp.day = decomposition.day;
			
			// Set date unit
			var dateUnit = '';
			if (dp.day != -1) {
				dateUnit = 'day';
			}
			else if (dp.month != -1) {
				dateUnit = 'month';
			}
			else if (dp.quarter != -1) {
				dateUnit = 'quarter';
			}
			else {
				dateUnit = 'year';
			}
			
			dp.dateUnit = dateUnit;
		}
		else {
			
			// No date filter is active, so we set a default date for semi selection
			dp.year = datePicker.earliest.year;
			dp.month = datePicker.earliest.month;
		}
		
		// If there is no active quarter or month we need to set a default value for it
		
		if (dp.quarter == -1 || dp.month == -1) {
			
			var firstMonthInYear = (dp.year == datePicker.earliest.year) ? datePicker.earliest.month : 0;
			var firstQuarterInYear = datePickerUtil.getQuarterFromMonth(firstMonthInYear);
			
			if (dp.quarter == -1) {
				dp.quarter = firstQuarterInYear;
			}
			
			if (dp.month == -1) {
				dp.month = firstMonthInYear;
			}
		}
		
		// util.showObject(datePicker.dp1);
	},
	
	initRelativeDpObject: function(df) {
		
		var dp = datePicker.dp3;
		
		if (df != '') {
			
			var pattern = /^(recent|last)([0-9]+)(year|quarter|month|day)(s)?$/;
			var result = df.match(pattern);
			// alert(result);
			dp.isActive = true;
			dp.type = result[1];
			dp.dateUnitCount = parseInt(result[2], 10);
			dp.dateUnit = result[3];
		}
		else {
			
			dp.isActive = false;
			dp.type = 'recent';
			dp.dateUnitCount = 1;
			dp.dateUnit = 'year';
		}
	},
		
	open: function() {
				
		if (datePicker.panel == null) {
			// Initialize the date picker
			datePicker.init();
		}
		
		// Get date filter info
		var dfInfo = reportInfo.dateFilterInfo;
		
		// util.showObject(dfInfo);
		
		// return false;
		
		// Check active dateFilter state in report, the date filter could be:
		// a.) A single absolute date filter --> 14jan2007
		// b.) An absolute date range --> 14jan2007-25jul2007
		// c.) A relative date filter --> 3days
		// d.) Multiple absolute date filters (i.e. upon zoom on date items) --> 14jan2007,16jan2007,1feb2007
		
		// Note, the date picker can't handle multiple dates as in d.), in this case we ignore the date filter
		
		// ToDo,
		// a.) We must validate for a valid pattern
		// b.) We must validate any absolute date filter if it is within the given earliest and latest date range
		
		var df = reportInfo.dateFilter;
		var decomposition1 = null;
		var decomposition2 = null;
		var relativeDf = '';
		// var df1 = '';
		// var df2 = '';
		// var df3 = '';
		var defaultDpName = 'dp1';
		// var dfPattern = '';
		
		if (dfInfo.isGlobalDateFilter && dfInfo.isValidDateFilterSyntax) {
			
			if (!dfInfo.isRelativeDateFilter) {
				
				dateType = dfInfo.dateType;
				// We ihnore multiple dates!
				if (dateType != 'multiple_dates' && !dfInfo.isOutOfRange) {
					
					decomposition1 = dfInfo.decomposition1;
					
					// Note, the date_type 'date_range' in date filter info
					// doesn't mean that we actualy have a date_filter with a date range
					// because a single df also may result in a date range, i.e. mar2007
					// So we check if the date filter contains a dash
					
					if (dateType == 'date_range' && (df.indexOf('-') != -1)) {
						// The date filter actually describes a date range
						decomposition2 = dfInfo.decomposition2;
						defaultDpName = 'dp2';
					}
				}
			}
			else {
				
				// This is a relative date filter,
				// check if we support the relative pattern in the date picker
				df = df.replace(/\s/g, '');
				df = df.toLowerCase();
				var relativeDfPattern = /^(recent|last)[0-9]+(year|quarter|month|day)(s)?$/;
				
				if (relativeDfPattern.test(df)) {
					// alert('relativeDfPattern MATCH');
					relativeDf = df;
					defaultDpName = 'dp3';
				}
			}
		}
		
		// Init dp1 and dp2 object
		datePicker.initCalendarDpObject('dp1', decomposition1);
		datePicker.initCalendarDpObject('dp2', decomposition2);
		datePicker.initRelativeDpObject(relativeDf);
		
		// Init relative picker display
		
		// util.showObject(datePicker.relative);
		datePicker.relative.applyDateSelection();
		
		datePicker.setPicker(defaultDpName);
		
		// Show fixed date info
		util.showE('dp:fixed_date_info', dfInfo.isFixedDate);
	
		datePicker.panel.open();
	},
	
	close: function() {
		
		datePicker.panel.close();
	},

	setPickerByEvent: function() {
		
		var id = this.id;
		var dat = id.split(':');
		var dpName = dat[1];
		
		datePicker.setPicker(dpName);
	},
	
	setPicker: function(dpName) {
		
		datePicker.setPickerButtons(dpName);
		datePicker.activePicker = dpName;
		
		util.hideE(['dp:calendar_picker', 'dp:relative_picker']);
		
		var innerPickerContainer = util.getE('dp:inner_pickers_container');
		
		if (dpName != 'dp3') {
			
			// Init calendar date picker
			
			// Reset inner picker container width to auto (fixed width is only used for relative picker!
			innerPickerContainer.style.width = 'auto';
			
			datePicker.calendar.init(datePicker[dpName]);
			util.showE('dp:calendar_picker');
		}
		else {
			
			// Init relative date picker
			
			// Get size of calendar picker, then set relative picker width
			// so that the GUI does not shrink for the smaller relative picker.
			// Note, the height is already set by a vertical spacer column in both pickers.
		
			// var relativePicker = util.getE('dp:relative_picker');
			// var calendarPicker =  = util.getE('dp:calendar_picker');
			
			util.showE('dp:calendar_picker');
			var containerRegion = yDom.getRegion('dp:inner_pickers_container');
			util.hideE('dp:calendar_picker');
			
			innerPickerContainer.style.width = containerRegion.width + 'px';
			// relativePicker.style.height = (containerRegion.height - 1) + 'px';
			// relativePicker.style.display = 'block';
			
			util.showE('dp:relative_picker');
		}
		
		datePicker.updatePickedDateDisplay();
	},
	
	setPickerButtons: function(activeDpType) {
		
		// alert('activeDpType: ' + activeDpType);
		
		var dps = ['dp1', 'dp2', 'dp3'];
		
		for (var i = 0; i < dps.length; i++) {
			
			var dpName = dps[i];
			var isActive = (activeDpType == dpName);
			var a = util.getE('dp:' + dpName + ':btn');
			a.className = !isActive ? 'btn-30' : 'btn-30-pressed';
			
			if (isActive) {
				a.blur();
			}
			
			// alert('a.className: ' + a.className);
		}
	},
	
	
	//
	//
	//
	// Calendar date picker
	//
	//
	//
	
	calendarPickerItemActivated: function(evt) {
		
		// alert('calendarPickerItemActivated()');
		
		var element = evt.target || evt.srcElement;
		var elementId = element.id;
		
		// alert('calendarPickerItemActivated() - elementId: ' + elementId);
		// A valid event must contain the ':cal:' string within the callee ID
		
		if (elementId != '') {
			
			element.blur();
			
			var dat = elementId.split(':');
			
			if (dat[1] == 'cal') {
				
				var dateUnit = dat[2];
				var dateNum = parseInt(dat[3], 10);
				
				var elementClassName = element.className;
				
				if (element.className != 'no-log-data') {
				
					datePicker.calendar.itemActivated(dateUnit, dateNum);
					datePicker.updatePickedDateDisplay();
				}
			}
		}
	},
	
	
	//
	//
	//
	// Relative date picker
	//
	//
	//
	
	relativePickerItemActivated: function(evt) {
		
		var element = evt.target || evt.srcElement;
		var elementId = element.id;
		
		if (elementId != '') {
			
			var dat = elementId.split(':');
		
			if (dat.length == 4) {
				
				element.blur();
				
				var idPropertyName = dat[2];
				
				// alert('idPropertyName: ' + idPropertyName);
				
				var propertyName = '';
				if (idPropertyName == 'type') {
					propertyName = 'type';
				}
				else if (idPropertyName == 'date_unit_count') {
					propertyName = 'dateUnitCount';
				}
				else {
					propertyName = 'dateUnit';
				}
				
				var propertyValue = (propertyName == 'dateUnitCount') ? parseInt(dat[3], 10) : dat[3];
				
				datePicker.dp3.isActive = true;
				datePicker.dp3[propertyName] = propertyValue;
				
				datePicker.relative.applyDateSelection();
			}
		}
		
		datePicker.updatePickedDateDisplay();
	},
	
	relativePickerDateUnitCountInput: function() {
		
		// User typed a number into relative date picker date display input field
		// alert('relativePickerNumberItemActivated() - this.value: ' + this.value);
		
		var value = util.trim(this.value);
		if (util.isInteger(value, 1)) {
			
			datePicker.dp3.dateUnitCount = value;
			datePicker.relative.applyDateSelection();
		}
	},
	
	//
	//
	//
	// Clear picker date (valid for calendar and relative picker)
	//
	//
	//
	
	clearPickerDate: function() {
		
		var dpName = datePicker.activePicker;
		
		// alert('clearPickerDate() - activeDpName: ' + dpName);
		
		if (dpName != 'dp3') {
			
			var dp = datePicker[dpName];
			dp.dateUnit = '';
			dp.day = -1;
			datePicker.calendar.init(dp);
		}
		else {
			
			datePicker.dp3.isActive = false;
			datePicker.relative.applyDateSelection();
		}
		
		datePicker.updatePickedDateDisplay();
	},
	
	//
	//
	// Picked date display
	//
	//
		
	updatePickedDateDisplay: function() {
				
		// Update the finally picked date display
		
		var dpName = datePicker.activePicker;
		
		// util.showObject(dp);
		
		var text = '';
		var entireDateRangeMsg = langVar('lang_stats.date_picker.entire_date_range');
		var showCalendarDateDisplay = true;
		
		util.hideE(['dp:calendar_date_display', 'dp:relative_date_display']);
		
		switch (dpName) {
			
			case 'dp1':
			
				text = (datePicker.dp1.dateUnit != '') ? datePicker.assembleCalendarDateLabel('dp1') : entireDateRangeMsg;
				break;
				
			case 'dp2':
			
				if (datePicker.dp2.dateUnit != '') {
					
					var startText = (datePicker.dp1.dateUnit != '') ? datePicker.assembleCalendarDateLabel('dp1') : langVar('lang_stats.date_picker.earliest_date');
					text = startText + ' - ' + datePicker.assembleCalendarDateLabel('dp2');
				}
				else {
					text = entireDateRangeMsg;
				}
			
				break;
				
			case 'dp3':
			
				if (datePicker.dp3.isActive) {
					
					showCalendarDateDisplay = false;
					
					var dp = datePicker.dp3; 
					
					var typeLabel = (dp.type == 'recent') ? langVar('lang_stats.date_picker.recent') : langVar('lang_stats.date_picker.last');
					var dateUnitLabelsPlural = datePicker.dateUnitLabelsPlural;
					var dateUnitLabel = dateUnitLabelsPlural[dp.dateUnit];
					
					util.updateT('dp:relative_date_display:type', typeLabel);
					util.setF('dp:relative_date_display:date_unit_count', dp.dateUnitCount);
					util.updateT('dp:relative_date_display:date_unit', dateUnitLabel);
				}
				else {
					// Shows the calendar display!
					text = entireDateRangeMsg;
				}
				
				break;
		}
		
		if (showCalendarDateDisplay) {
			
			util.updateT('dp:calendar_date_display', text);
			util.showE('dp:calendar_date_display');
		}
		else {
			
			util.showE('dp:relative_date_display');
		}
	},
	
	assembleCalendarDateLabel: function(dpName) {
		
		var dp = datePicker[dpName];
		var dateUnit = dp.dateUnit;
		
		// alert('dpName: ' + dpName);
		// util.showObject(dp);
		
		var dpDisplay;
		
		if (dateUnit == 'year') {
			
			dpDisplay = dp.year;
		}
		else if (dateUnit == 'quarter') {
			
			dpDisplay = lang.quarterShort + dp.quarter + '/' + dp.year;
		}
		else {
			
			var month = lang.monthsShort[dp.month];
			
			dpDisplay = month + '/' + dp.year;
			
			// alert('dateUnit: ' + dateUnit + '\ndpDisplay: ' + dpDisplay);
			
			if (dateUnit == 'day') {
				
				var day = (dp.day < 10) ? '0' + dp.day : dp.day;
				
				dpDisplay = day + '/' + dpDisplay;
			}
		}
		
		return dpDisplay;
	},
	
	
	//
	//
	//
	// Apply date filter
	//
	//
	//
	
	applyDateFilter: function() {
		
		var dpName = datePicker.activePicker;
		var df = '';
		
		switch (dpName) {
			
			case 'dp1':
			
				if (datePicker.dp1.dateUnit != '') {
					df = datePickerUtil.convertDpObjectToDf('dp1');
				}
		
				break;
			
			case 'dp2':
			
				if (datePicker.dp2.dateUnit != '') {
					
					var dp1 = datePicker.dp1;
					var dp2 = datePicker.dp2;
					
					// Verify if dp1 is set or not. If it is not set then set it now.
					if (dp1.dateUnit == '') {
						
						// No start date is set, so we have to set one
						
						var earliest = datePicker.earliest;
						
						dp1.dateUnit = 'day';
						dp1.year = earliest.year;
						dp1.month = earliest.month;
						dp1.day = earliest.day;
					}
					
					var df1 = datePickerUtil.convertDpObjectToDf('dp1');
					var df2 = datePickerUtil.convertDpObjectToDf('dp2');
					
					var startTime;
					var endTime;
					
					if (dp1.dateUnit == dp2.dateUnit) {
						
						// dp1 and dp2 have the same dateUnit
						
						if (df1 != df2) {
							
							// Check if start date is smaller than end date, else invert the two
							startTime = datePickerUtil.convertDpObjectToTime('dp1');
							endTime = datePickerUtil.convertDpObjectToTime('dp2');
							
							if (startTime <= endTime) {
								df = df1 + '-' + df2;
							}
							else {
								// invert start/end date
								df = df2 + '-' + df1;
							}
						}
						else {
							
							// We have the same date in dp1 and dp2, use only dp1
							df = df1;
						}
					}
					else {
						
						// dp1 and dp2 have a different dateUnit
						// We need to check for a valid date, the startTime must be smaller or equal the endTime.
						
						startTime = datePickerUtil.convertDpObjectToTime('dp1');
						endTime = datePickerUtil.convertDpObjectToTime('dp2');
						
						if (startTime <= endTime) {
							
							df = df1 + '-' + df2;
						}
						else {
							
							alert(langVar('lang_stats.date_picker.invalid_date_range_msg'));
							return false;
						}
					}
				}
			
				break;
			
			case 'dp3':
			
				var dp3 = datePicker.dp3;
			
				if (dp3.isActive) {
					
					var relativeType = dp3.type;
					var dateUnitCount = dp3.dateUnitCount;
					df = relativeType + dateUnitCount + dp3.dateUnit;
					
					if (dateUnitCount > 1) {
						// Use plural for date unit
						df += 's';
					}
				}
			
				break;
		}
		
		//alert('applyDateFilter() - df: ' + df);
		// return false;
		
		datePicker.close();
		newReport.setDateFilter(df);
	},	
	
	//
	//
	//
	// Date picker builder utilities
	//
	//
	//
	
	build: function() {
		
		
		var formContainer = util.getE('dp:form_container');
		var buttonsContainer = util.createE('div', {paddingTop:'9px',paddingLeft:'9px'});
		var outerPickersContainer = util.createE('div', {className:'dp-outer-container'}); // Includes the header bar which indicates the date
		var innerPickersContainer = util.createE('div', {id:'dp:inner_pickers_container', className:'dp-inner-container'});
		
		datePicker.buildMainButton(buttonsContainer, 'dp:dp1:btn', langVar('lang_stats.date_picker.date_or_start_date'));
		datePicker.buildMainButton(buttonsContainer, 'dp:dp2:btn', langVar('lang_stats.date_picker.end_date'));
		datePicker.buildMainButton(buttonsContainer, 'dp:dp3:btn', langVar('lang_stats.date_picker.relative_date'));
		
		// We need to clear the buttonsContainer
		var buttonsClearanceDiv = util.createE('div', {className:'clearance'});
		var spaceText = util.createT('&nbsp;');
		
		datePicker.buildHeaderBar(outerPickersContainer);
		util.chainE(outerPickersContainer, innerPickersContainer);
		
		// datePicker.buildStartEndDatePicker(pickersContainer);
		
		util.chainE([formContainer, [buttonsContainer, [buttonsClearanceDiv, spaceText]], outerPickersContainer]);
		
		// var tbody = util.getE('date_picker:container');
		// datePicker.createMainColumns(tbody);
	},
	
	
	buildMainButton: function(container, id, label) {
		
		var a = util.createE('a', {id:id, href:'javascript:;', className:'btn-30'});
		var text = util.createT(label); 
		util.chainE(container, a, text);
	},
	
	buildHeaderBar: function(container) {
	
		var table = util.createE('table', {className:'dp-header-bar', cellSpacing:'0'});
		var tbody = util.createE('tbody');
		var tr = util.createE('tr');
		var th = util.createE('th', {id:'dp:date_display'});
		
		var calendarDateDiv = util.createE('div', {id:'dp:calendar_date_display', display:'none'});
		var relativeDateDiv = util.createE('div', {id:'dp:relative_date_display', display:'none'});
		var relativeDateTypeSpan = util.createE('span', {id:'dp:relative_date_display:type'});
		var realtiveDateNumberInput = util.createE('input', {id:'dp:relative_date_display:date_unit_count', className:'dp-relative-date-unit-count', type:'text', value:''})
		var relativeDateUnitSpan = util.createE('span', {id:'dp:relative_date_display:date_unit'});
		
		util.chainE([relativeDateDiv, relativeDateTypeSpan, realtiveDateNumberInput, relativeDateUnitSpan]);
		util.chainE([th, calendarDateDiv, relativeDateDiv]);
		
		var td = util.createE('td');
		var a = util.createE('a', {id:'dp:clear_picker_date:btn', href:'javascript:;'});
		var text = util.createT(langVar('lang_stats.btn.clear'));
		
		util.chainE(td, a, text);
		
		util.chainE(container, table, tbody, [tr, th, td]);
	}
}
